import React, { useEffect, useState } from "react";
import { PlusOutlined } from "@ant-design/icons";
import { Button, Image, Upload } from "antd";
import type { GetProp, UploadFile, UploadProps } from "antd";
import { isEqual } from "lodash";
import { composeImgUrl } from "@/config/util";

type FileType = Parameters<GetProp<UploadProps, "beforeUpload">>[0];

interface ImageUploadProps {
  onChange: (params?: string) => void;
  value: string;
  placeholder?: string;
  templateUrl?: string;
}
const getBase64 = (file: FileType): Promise<string> =>
  new Promise((resolve, reject) => {
    const reader = new FileReader();
    reader.readAsDataURL(file);
    reader.onload = () => resolve(reader.result as string);
    reader.onerror = (error) => reject(error);
  });

const ImageUpload: React.FC<ImageUploadProps> = (props) => {
  const { value, onChange, placeholder, templateUrl } = props;

  const [previewOpen, setPreviewOpen] = useState(false);
  const [previewImage, setPreviewImage] = useState("");
  const [fileList, setFileList] = useState<any[]>([]);

  useEffect(() => {
    if (value && value.length !== 0) {
      const list = value.split(",");
      const temp = list.map((v, i) => {
        return {
          uid: (i + 1).toString(),
          name: v,
          status: "done",
          url: composeImgUrl(v),
        };
      });

      if (temp && temp?.length > 0 && !isEqual(temp, fileList)) {
        setFileList(temp);
      }
    }
  }, [value]);

  const handlePreview = async (file: UploadFile) => {
    if (!file.url && !file.preview) {
      file.preview = await getBase64(file.originFileObj as FileType);
    }

    setPreviewImage(file.url || (file.preview as string));
    setPreviewOpen(true);
  };

  const handleChange: UploadProps["onChange"] = ({
    fileList: newFileList,
    file,
  }) => {
    const { status } = file;
    setFileList(newFileList);
    if (status === "done") {
      const imgStr = newFileList.map((item) => {
        return item.response.data;
      });

      onChange(imgStr.join(","));
    }
  };

  const uploadButton = (
    <button style={{ border: 0, background: "none" }} type="button">
      <PlusOutlined />
      <div style={{ marginTop: 8 }}>{placeholder ?? "上传图片"}</div>
    </button>
  );
  const onRemove = () => {
    console.log("onRemove");
    setFileList([]);
    onChange(undefined);
  };
  // console.log(templateUrl);
  return (
    <>
      <div>{templateUrl && <Button>下载模版</Button>}</div>
      <Upload
        action="/apis/oss/upload?bucketName=cgtf"
        listType="picture-card"
        fileList={fileList as any}
        onPreview={handlePreview}
        maxCount={1}
        accept="image/*"
        onChange={handleChange}
        onRemove={onRemove}
      >
        {fileList.length >= 1 ? null : uploadButton}
      </Upload>
      {previewImage && (
        <Image
          wrapperStyle={{ display: "none" }}
          preview={{
            visible: previewOpen,
            onVisibleChange: (visible) => setPreviewOpen(visible),
            afterOpenChange: (visible) => !visible && setPreviewImage(""),
          }}
          src={previewImage}
        />
      )}
    </>
  );
};

export default ImageUpload;
