import "reflect-metadata";
import BaseFormComponent from "../entity/base";
import {
  PropInfo,
  PropInfoMap,
  Type,
  FormComponentRegistryInfo,
} from "../interfaces";
import { getOrDefineMap, getOrDefineField } from "../utils";
import { Components } from "../registry";

const PROPERTY = Symbol("Property");
const COMPONENT = Symbol("Component");

/**
 *
 */
export function getPropMap(target: Type): PropInfoMap {
  return getOrDefineMap(target, PROPERTY);
}

/**
 *
 * 属性定义
 * 定义一个组件的属性
 *
 * ```
 * @Component()
 * class Button extends FrameworkElement {
 *      @Property({  })
 *      foreground!: string;
 * }
 *
 * ```
 * */
export function Property(info: PropInfo): PropertyDecorator {
  const prop = (target: BaseFormComponent, key: string | symbol): void => {
    if (typeof key === "symbol") throw new Error("Symbol prop is not allowed");

    const type = info.type;
    // reflect 获取不到原始数据类型，只有换种途径
    // ?? (Reflect.getMetadata("design:type", target, key) as Type)

    const rtInfo = {
      ...info,
      type,
    };
    getPropMap(target.constructor)[key as keyof BaseFormComponent] =
      Object.freeze(rtInfo);
  };
  return prop as PropertyDecorator;
}

/**
 * 获取表单组件信息
 */
export function getCompInfo(target: Type<any>): any {
  return getOrDefineField(target, COMPONENT, {} as any);
}

/**
 * 类装饰器
 * 定义一个组件，并将组件注册到全局的 `Components`
 * ```
 * @Component({ abstract: true })
 * class Button extends FrameworkElement {
 *   //...xxxx
 * }
 * ```
 * */
export function Component(compInfo: FormComponentRegistryInfo): ClassDecorator {
  const deco = (target: any): void => {
    const base = Object.getPrototypeOf(target) as Type<any>;
    const baseInfo = getCompInfo(base);

    getOrDefineField(target, COMPONENT, {
      ...baseInfo,
      ...compInfo,
    });
    if (compInfo.name) Components.add(target);
  };
  return deco as ClassDecorator;
}
