import { cloneDeep } from "lodash";
import BaseFormComponent from "./entity/base";
import { v4 } from "uuid";

/**
 * 在类上注册字段，或获取已经注册的字段
 */
export function getOrDefineField<U, T extends Function>(
  target: T,
  key: symbol,
  def: U
): U {
  if (!Object.prototype.hasOwnProperty.call(target, key)) {
    if (def === undefined) return undefined as unknown as U;
    Object.defineProperty(target, key, {
      configurable: true,
      value: def,
    });
  }
  return Reflect.get(target, key) as U;
}

/**
 * 在类上注册Map字段，或获取已经注册的Map字段，并自动集成基类信息
 */
export function getOrDefineMap<U, T extends Function>(
  target: T,
  key: symbol
): Record<string, U> {
  if (!Object.prototype.hasOwnProperty.call(target, key)) {
    Object.defineProperty(target, key, {
      configurable: true,
      value: { ...Reflect.get(target, key) },
    });
  }
  return Reflect.get(target, key) as Record<string, U>;
}

/**
 * 通过configList构建分组
 */
export function buildListByGroupId(target: Array<BaseFormComponent>) {
  const groupList: Array<{
    title: string;
    list: Array<BaseFormComponent>;
  }> = [];

  const groupMap: { [key: string]: Array<BaseFormComponent> } = {};

  target.forEach((formConfig) => {
    if (!groupMap[formConfig?.groupId]) {
      groupMap[formConfig?.groupId] = [];
    }
    groupMap[formConfig?.groupId].push(formConfig);
  });

  for (const groupTitle in groupMap) {
    if (groupMap.hasOwnProperty(groupTitle)) {
      groupList.push({
        title: groupTitle,
        list: groupMap[groupTitle],
      });
    }
  }

  return groupList;
}
/** 请确保所有元素是按照正确的顺序排列，不然会存在问题 */
export function buildTreeByGroup(target: Array<BaseFormComponent>) {
  const output = [] as Array<
    BaseFormComponent & { children?: Array<BaseFormComponent> }
  >;

  target.forEach((item) => {
    const newItem = cloneDeep(item) as any;
    if (item.componentType === "Branch") {
      newItem.children = item.componentOption.children;
    }
    output.push(newItem);
  });

  return output;
}
/** 生成uuid */
export function generateUUID() {
  let id = "";
  do {
    id = v4().replace(/-/g, ""); // 去掉连字符
  } while (/\d/.test(id.charAt(0))); // 如果第一个字符是数字，重新生成
  return id;
}

export function buildTreeByGroupName(target: any[]) {
  // debugger;
  const groupList = target.reduce((pre, cur) => {
    if (cur.groupName in pre) {
      if ("child" in pre[cur.groupName]) {
        pre[cur.groupName]["child"].push(cur);
      } else pre[cur.groupName]["child"] = [cloneDeep(pre[cur.groupName]), cur];
    } else {
      pre[cur.groupName] = cloneDeep(cur);
    }
    return pre;
  }, {} as any);
  return groupList;
}

/** 反序列化规则 */
export function inversionSerializationRule(inputStr: string) {
  let value = JSON.parse(inputStr);

  if (Array.isArray(value) && value.length > 0) {
    value[0].pattern = eval(value[0].pattern);
  }

  return value;
}
