import BasicModal from "@/components/BasicModal";
import styles from "./index.module.css";
import applyIcon from "@/assets/images/form/apply.png";
import { Button, Form, message } from "antd";
import GroupCard from "./components/groupCard";
import {
  buildListByGroupId,
  inversionSerializationRule,
} from "@/customForm/utils";
import RenderComponent from "../../../components/FormComponentRender/renderComponent";
import BaseFormComponent from "@/customForm/entity/base";
import { OnlyProperties } from "@/customForm/interfaces";
import { useEffect, useMemo, useState } from "react";
import { useModalConfigContext } from "../Main";
import service from "@/service";
import useUserStore from "@/store/useUserStore";

const SubmitApplication = () => {
  const [form] = Form.useForm();
  const { modalOption, setModalOption } = useModalConfigContext();
  const [visible, setVisible] = useState(false);
  const [dataSource, setDataSource] = useState<any>({});
  const { userInfo } = useUserStore();
  const [applicationConfig, setApplicationConfig] = useState<any>();
  const config = useMemo(() => {
    const matterContentList = dataSource?.matterContentList;
    let groupList = [] as any;
    let configList = [] as any;
    if (matterContentList && applicationConfig?.examineKeys) {
      let temp = matterContentList
        .sort((pre: any, cur: any) => pre.sort - cur.sort)
        .map((item: any) => {
          return {
            ...item,
            componentOption: item.otherSet ? JSON.parse(item.otherSet) : {},
            key: item.fieldKey,
            validationRules: inversionSerializationRule(item.validationRules),
          };
        });
      /** 过滤需要处理的内容 */
      if (applicationConfig?.examineKeys !== "all") {
        const keys = applicationConfig?.examineKeys?.split(",") ?? [];
        temp = temp.filter((item: any) => keys.includes(item.fieldKey));
      }
      temp.forEach((item: OnlyProperties<BaseFormComponent>) => {
        if (item.componentType === "Branch") {
          groupList.push(item);
        } else {
          configList.push(item);
        }
      });
      return {
        groupList: groupList,
        configList: buildListByGroupId(configList),
      };
    }
    return {
      groupList: [],
      configList: [],
    };
  }, [dataSource, applicationConfig]);

  const getGroupName = (key: string) => {
    return config.groupList.find((v: any) => v.key === key)?.label ?? "";
  };

  useEffect(() => {
    if (modalOption.show) {
      setVisible(true);
    } else {
      form.resetFields();
      setVisible(false);
    }
  }, [modalOption]);

  useEffect(() => {
    const id = modalOption.metaData?.id;
    if (id) {
      service.settingService.getTemplateConfig(id).then((res) => {
        if (res.code === 200) {
          setDataSource(res.data);
          service.applyService
            .getApplicationNodeConfig(res.data.flowProcessId)
            .then((res1) => {
              if (res1.code === 200) {
                setApplicationConfig(res1.data);
              }
            });
        }
      });
    }
  }, [modalOption.metaData]);

  const onSubmit = () => {
    form.submit();
  };

  const closeHandler = () => {
    setVisible(false);
    setModalOption({ show: false, metaData: {} });
  };
  const onFinish = async (data: any) => {
    const metaData = {
      thingId: modalOption.metaData.id,
      name: modalOption.metaData.thingName,
      loginName: userInfo.username,
      applicantType: "自然人",
      remake: "",
    };
    let forms = [];
    const matterContentList = dataSource?.matterContentList;

    for (const key in data) {
      const itemConfig = matterContentList.find((v: any) => v.fieldKey === key);
      if (itemConfig) {
        forms.push({
          reportName: userInfo.mobile,
          contentId: key,
          content: data[key],
          downFlag:
            itemConfig.componentType === "FileUpload" ||
            itemConfig.componentType === "ImgUpload"
              ? true
              : false,
        });
      }
    }
    try {
      const { data: formId, code } = await service.applyService.saveApplyItem({
        ...metaData,
        reportContents: forms,
      });
      if (code !== 200) {
        throw new Error();
      }
      const params = new FormData();
      params.append("businessId", formId);
      params.append("applicant", userInfo.id);
      params.append("processId", modalOption.metaData.processId);

      const { data: taskId, code: code2 } =
        await service.applyService.startExamine(params);
      if (code2 !== 200) {
        throw new Error();
      }

      const params2 = new FormData();
      params2.append("taskId", taskId);
      params2.append("user", userInfo.id);
      params2.append("approveComment", "已发起");

      const { code: code3 } = await service.applyService.completeNoAuthExamine(
        params2
      );
      if (code3 !== 200) {
        throw new Error();
      }

      message.success("提交成功");
      setModalOption({
        show: false,
        metaData: {},
      });
    } catch {
      message.error("提交失败");
    }
  };
  return (
    <BasicModal
      visible={visible}
      width={900}
      height={792}
      onClose={closeHandler}
      contentStyle={{ borderRadius: 10 }}
    >
      <div className={styles.container}>
        <div className={styles.header}>
          <img src={applyIcon} style={{ width: 46, height: 46 }} alt=""></img>
          <div className={styles.headerTitle}>
            申请·{modalOption.metaData.thingName}
          </div>
        </div>
        <div className={styles.modalBody}>
          <div className={styles.cardWrapper}>
            <Form
              form={form}
              colon={false}
              labelCol={{ span: 6 }}
              wrapperCol={{ span: 18 }}
              onFinish={onFinish}
            >
              {config.configList.map((item: any) => {
                return (
                  <GroupCard title={getGroupName(item.title)}>
                    {item.list.map((formItem: any) => {
                      return (
                        <Form.Item
                          label={formItem.label}
                          name={formItem.key}
                          rules={[
                            {
                              required: formItem.componentOption?.require,
                              message: `请填写${formItem?.label}`,
                            },
                            ...formItem.validationRules,
                          ]}
                        >
                          <RenderComponent
                            config={formItem.componentOption}
                            componentType={formItem.componentType}
                          ></RenderComponent>
                        </Form.Item>
                      );
                    })}
                  </GroupCard>
                );
              })}
            </Form>
          </div>
          <div style={{ float: "right", paddingTop: 34 }}>
            <Button
              type="primary"
              size="large"
              onClick={onSubmit}
              style={{ width: 150, height: 50 }}
            >
              提交
            </Button>
          </div>
        </div>
      </div>
    </BasicModal>
  );
};

export default SubmitApplication;
